package pro.litvinovg.libreoffice.metadata;

import java.util.ArrayList;
import java.util.Iterator;

import org.json.JSONException;
import org.json.JSONObject;

import com.sun.star.beans.PropertyVetoException;
import com.sun.star.beans.UnknownPropertyException;
import com.sun.star.beans.XPropertySet;
import com.sun.star.lang.IllegalArgumentException;
import com.sun.star.lang.WrappedTargetException;
import com.sun.star.text.XText;
import com.sun.star.text.XTextCursor;
import com.sun.star.text.XTextField;
import com.sun.star.text.XTextRange;
import com.sun.star.uno.UnoRuntime;
import static pro.litvinovg.libreoffice.metadata.Document.*;
public class OutlineElement {

	private String name;
	private ArrayList<MetadataElement> metadata = null;

	public String getName() {
		return name;
	}

	private XTextRange textRange;
	private XTextRange annotationStart = null;
	private XTextRange annotationEnd = null;

	public OutlineElement(XTextRange textRange) {
		this.textRange = textRange;
		this.name = textRange.getString();
		this.metadata = new ArrayList<MetadataElement>();
		annotationStart = textRange.getStart();
	}
	public void setAnnotationEnd(XTextRange anchor) {
		this.annotationEnd = anchor;
	}
	
	public void readMetadata(String data) {
		try {
  		JSONObject json = new JSONObject(data);
  		Iterator<String> names = json.keys();
  		while (names.hasNext()) {
				String metaName = names.next();
				String metaValue = (String) json.get(metaName);
				metadata.add(new MetadataElement(metaName, metaValue));
			}
		} catch (JSONException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
private String metadataToString() {
    JSONObject json = new JSONObject();
    for (MetadataElement element : metadata) {
    	json.put(element.getName(), element.getValue());	
    }
		return json.toString();
	}
public void writeMetadata(Object annotation) {
	XPropertySet properties = UnoRuntime.queryInterface(XPropertySet.class, annotation);
	XText startText = annotationStart.getText();
	try {
		properties.setPropertyValue("Author", METADATA_EXTENSION);
		properties.setPropertyValue("Content", metadataToString());
		XTextRange annotationRange = null;
		if (startText.equals(annotationEnd.getText())){
			XTextCursor cursor = startText.createTextCursorByRange(annotationStart);
			cursor.gotoRange(annotationEnd, true);
			 annotationRange = UnoRuntime.queryInterface(XTextRange.class, cursor);
		} else {
			annotationRange = textRange;
		}
		XTextField textField = UnoRuntime.queryInterface(XTextField.class, annotation);
		startText.insertTextContent( annotationRange, textField, true );
	} catch (IllegalArgumentException e) {
		e.printStackTrace();
	} catch (UnknownPropertyException e) {
		e.printStackTrace();
	} catch (PropertyVetoException e) {
		e.printStackTrace();
	} catch (WrappedTargetException e) {
		e.printStackTrace();
	}
}
	


}
